const error = require('http-errors');
const Promise = require('bluebird');

const logger = require('../../../utils/logger');
const { db } = require('../../../routes/global');
const { formatWinAnchor, _prepareUpdate, tryParseJSON } = require('../../../routes/win_anchor/utils');


const insertSync = db.makeInsert('win_anchor_sync');
const updateSync = db.makeUpdateByUid('win_anchor_sync');
const updateByUid = db.makeUpdateByUid('win_anchors');
const findByUid = db.makeFindByUid('win_anchors');

function formatWinAnchorSync(anchor) {
    if (anchor) {
        const obj = Object.assign({}, formatWinAnchor(anchor));
        obj.sync_slot = anchor.sync_slot ? tryParseJSON(obj.sync_slot) : null;
        obj.addr = anchor.node_id
            ? anchor.node_id.toString(16).padStart(4, '0').toUpperCase()
            : null;
        return obj;
    }
    return anchor;
}

// static synchronization prototype
async function getWinAnchorSync(project, company, uid) {
    let sql;
    const args = [];
    if (!company && !project) {
        Promise.resolve([]);
    } else if (!company && project) {
        sql = 'SELECT w.*, s.sync_slot FROM view_win_anchors w left join win_anchor_sync s on s.uid=w.uid WHERE project_uid = ?';
        args.push(project.uid);
    } else if (!project) {
        sql = 'SELECT w.*, s.sync_slot FROM view_win_anchors w left join win_anchor_sync s on s.uid=w.uid WHERE company_uid = ?';
        args.push(company.uid);
    } else {
        sql = 'SELECT w.*, s.sync_slot FROM view_win_anchors w left join win_anchor_sync s on s.uid=w.uid WHERE project_uid = ? AND company_uid = ?';
        args.push(project.uid, company.uid);
    }

    if (uid) {
        sql += ' AND w.uid = ?';
        args.push(uid);
    }
    return (await db.queryAsync(sql, args)).map(anchor => formatWinAnchorSync(anchor));
}

async function updateWinAnchorSync(uid, _data, project, company) {
    const winAnchor = await findByUid(uid);
    if (!winAnchor) {
        throw error(404, `Win Anchor "${uid}" not found`);
    }
    const data = _prepareUpdate(_data);
    if ('type' in _data) {
        delete data.win_type_id;
        delete data.type;
    }
    const { sync_slot } = _data;
    delete data.sync_slot;
    const promises = [updateByUid(data, uid)];
    if (sync_slot) {
        if (sync_slot.addr !== 'FFFF') {
            // validate if anchor with sync_slot.addr exist
            const node_id = parseInt(sync_slot.addr, 16);

            const foundAnchor = await db.queryAsync('select * from view_win_anchors where node_id=?;', [node_id]).get(0);

            if (!foundAnchor) {
                logger.error(`Not exists associated anchor with addr ${sync_slot.addr}`);
                throw error(422, `Not exists associated anchor with addr ${sync_slot.addr}`);
            }
        }

        const existsItem = await db.queryAsync('select * from win_anchor_sync where uid=?;', [uid]).get(0);
        if (existsItem) {
            const syncData = {
                sync_slot: JSON.stringify(sync_slot),
            };
            promises.push(updateSync(syncData, uid));
        } else {
            const syncData = {
                uid,
                sync_slot: JSON.stringify(sync_slot),
            };
            promises.push(insertSync(syncData));
        }
    }
    await Promise.join(...promises);

    return getWinAnchorSync(project, company, uid);
}

module.exports = {
    getWinAnchorSync,
    updateWinAnchorSync,
};
